@@ -1,7 +1,6 @@
 import logging
 import os
 from pathlib import Path
-import time
 
 import connexion
 
@@ -18,8 +17,6 @@ from flask_authorize import Authorize
 
 from .jwt_patch import flask_jwt_extended
 
-from flask_jwt_extended import get_current_user
-
 from .config import config
 from .template_filters import replace_empty
 from .setup_logging import setup_logging
@@ -45,6 +42,12 @@ datastore_path.mkdir(parents=True, exist_ok=True)
 setup_logging(datastore, options)
 logger = logging.getLogger("dashboard")
 
+# If there is no database we need to initialize!
+db_path = datastore_path / "seamm.db"
+if not db_path.exists():
+    logger.warning("The database does not exists, so forcing initialization")
+    options["initialize"] = True
+
 # Two of the Flask options cannot be reset, and should (apparently) be
 # handled with environment variables ... so if they are in the options
 # set the correct environment variables. Carefully!
@@ -75,8 +78,7 @@ cors = CORS()
 bootstrap = Bootstrap()
 
 jwt = flask_jwt_extended.JWTManager()
-authorize = Authorize(current_user=get_current_user)
-
+authorize = Authorize(current_user=flask_jwt_extended.get_current_user)
 moment = Moment()
 toolbar = DebugToolbarExtension()
 
@@ -84,6 +86,22 @@ db = SQLAlchemy()
 ma = Marshmallow()
 
 
+@jwt.user_lookup_loader
+def user_loader_callback(jwt_header, jwt_payload):
+    """Function for app, to return user object"""
+
+    if jwt_header:
+        from seamm_datastore.database.models import User
+
+        username = jwt_payload["sub"]["username"]
+        user = User.query.filter_by(username=username).one_or_none()
+
+        return user
+    else:
+        # return None / null
+        return None
+
+
 def create_app(config_name=None):
     """Flask app factory pattern
     separately creating the extensions and later initializing"""
@@ -137,14 +155,52 @@ def create_app(config_name=None):
         + app.config["SQLALCHEMY_DATABASE_URI"]
     )
 
-    conn_app.add_api("swagger.yml")
+    # Authorization configuration
+    app.config["AUTHORIZE_DEFAULT_PERMISSIONS"] = dict(
+        owner=["read", "update", "delete", "create", "manage"],
+        group=["read", "update"],
+        other=[""],
+    )
+    app.config["AUTHORIZE_ALLOW_ANONYMOUS_ACTIONS"] = True
 
+    # Set application to store JWTs in cookies.
+    app.config["JWT_TOKEN_LOCATION"] = ["cookies"]
+
+    # Set the cookie paths
+    # app.config["JWT_ACCESS_COOKIE_PATH"] = "/api"
+    app.config["JWT_REFRESH_COOKIE_PATH"] = "/api/auth/token/refresh"
+
+    # Cookie security
+    app.config["JWT_COOKIE_SECURE"] = False
+    app.config["JWT_COOKIE_CSRF_PROTECT"] = True
+    app.config["JWT_CSRF_ACCESS_PATH"] = "/api/"
+
+    conn_app.add_api("swagger.yml")
     db.init_app(app)
     with app.app_context():
-        if options["initialize"]:
-            logger.info("Removing all previous jobs from the database.")
+        from seamm_datastore.database.build import import_datastore, _build_initial
+
+        if options["initialize"] or config_name and config_name.lower() == "testing":
+            logger.warning("Removing all previous jobs from the database.")
             db.drop_all()
-        db.create_all()
+            db.create_all()
+            # Create database using other interface for consistency.
+            logger.info("Importing jobs...")
+            _build_initial(db.session, "default")
+
+        if config_name is None or not config_name.lower() == "testing":
+            # Log in as user running
+            import flask_authorize.plugin
+            from seamm_datastore.database.models import User
+
+            flask_authorize.plugin.CURRENT_USER = User.query.filter_by(id=2).one
+            temp_path = os.path.join(
+                os.path.expanduser(options["datastore"]), "projects"
+            )
+            logger.warning("Importing any jobs into the database.")
+            import_datastore(db.session, temp_path)
+
+            flask_authorize.plugin.CURRENT_USER = flask_jwt_extended.get_current_user
 
         from .routes.auth import auth as auth_blueprint
         from .routes.main import main as main_blueprint
@@ -175,114 +231,6 @@ def create_app(config_name=None):
     # jinja template
     app.jinja_env.filters["empty"] = replace_empty
 
-    # Authorization configuration
-    app.config["AUTHORIZE_DEFAULT_PERMISSIONS"] = dict(
-        owner=["read", "update", "delete", "create", "manage"],
-        group=["read", "update"],
-        other=[""],
-    )
-    app.config["AUTHORIZE_ALLOW_ANONYMOUS_ACTIONS"] = True
-
-    # Set application to store JWTs in cookies.
-    app.config["JWT_TOKEN_LOCATION"] = ["cookies"]
-
-    # Set the cookie paths
-    # app.config["JWT_ACCESS_COOKIE_PATH"] = "/api"
-    app.config["JWT_REFRESH_COOKIE_PATH"] = "/api/auth/token/refresh"
-
-    # Cookie security
-    app.config["JWT_COOKIE_SECURE"] = False
-    app.config["JWT_COOKIE_CSRF_PROTECT"] = True
-    app.config["JWT_CSRF_ACCESS_PATH"] = "/api/"
-
-    # Add some default roles to the dashboard
-    with app.app_context():
-        from .models import Role
-
-        role_names = ["user", "group manager", "admin"]
-
-        for role_name in role_names:
-            check = Role.query.filter_by(name=role_name).one_or_none()
-            if not check:
-                role = Role(name=role_name)
-                db.session.add(role)
-                db.session.commit()
-
-        # Add an admin group and user if not present
-        from .models import Group
-
-        name = "admin"
-        group = db.session.query(Group).filter_by(name=name).one_or_none()
-        if group is None:
-            group = Group(name=name)
-            db.session.add(group)
-            db.session.commit()
-
-        from .models import User
-
-        name = "admin"
-        user = db.session.query(User).filter_by(username=name).one_or_none()
-        if user is None:
-            admin_role = db.session.query(Role).filter_by(name="admin").one_or_none()
-
-            if admin_role is None:
-                admin_role = Role(name="admin")
-
-            user = User(username=name, password="admin", roles=[admin_role])
-            user.groups.append(group)
-            db.session.add(user)
-            db.session.add(admin_role)
-            db.session.add(group)
-            db.session.commit()
-
-        # And the current user also
-        item = Path.home()
-        # Get the group first
-        name = item.group()
-        group = db.session.query(Group).filter_by(name=name).one_or_none()
-        if group is None:
-            group = Group(name=name)
-            db.session.add(group)
-            db.session.commit()
-
-        # and now the user
-        name = item.owner()
-        user = db.session.query(User).filter_by(username=name).one_or_none()
-        if user is None:
-            admin_role = db.session.query(Role).filter_by(name="admin").one_or_none()
-
-            if admin_role is None:
-                admin_role = Role(name="admin")
-
-            user = User(username=name, password="default", roles=[admin_role])
-            user.groups.append(group)
-            db.session.add(user)
-            db.session.add(admin_role)
-            db.session.add(group)
-            db.session.commit()
-
-        # Add a default project if it does not exist.
-
-        from .models import Project
-
-        # Ensure that the directory exists
-        projects = datastore_path / "projects"
-        default = projects / "default"
-        default.mkdir(parents=True, exist_ok=True)
-
-        # Check if in DB
-        if (
-            db.session.query(Project)
-            .filter_by(name="default", path=str(default))
-            .one_or_none()
-        ) is None:
-            # Note this relies on the courrent user's id and group from above.
-            project = Project(
-                path=str(default), name="default", owner_id=user.id, group_id=group.id
-            )
-            db.session.add(project)
-            db.session.commit()
-
     logger.info("")
     logger.info("Final configuration:")
     logger.info(60 * "-")
@@ -290,29 +238,5 @@ def create_app(config_name=None):
         logger.info("\t{:>30s} = {}".format(key, value))
     logger.info("")
 
-    if not options["no_check"]:
-        # Ugly but avoids circular import.
-        from .models.import_jobs import import_jobs
-
-        t0 = time.perf_counter()
-        with app.app_context():
-            n_projects, n_added_projects, n_jobs, n_added_jobs = import_jobs(
-                str(projects)
-            )
-
-            db.session.commit()
-
-        t1 = time.perf_counter()
-        logger.info(
-            "Checked {} jobs and {} projects in {:.2f} s.".format(
-                n_jobs, n_projects, t1 - t0
-            )
-        )
-        if n_added_jobs > 0 or n_added_projects > 0:
-            logger.info(
-                "  added {} jobs and {} projects".format(n_added_jobs, n_added_projects)
-            )
-
     logger.info(f"{app.url_map}")
-
     return app
